"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const express_1 = __importDefault(require("express"));
const lodash_1 = require("lodash");
const async_1 = __importDefault(require("./async"));
const order_1 = __importDefault(require("./order"));
const schema_1 = __importDefault(require("./schema"));
const types_1 = require("./types");
const isPath = (arg) => (typeof arg === 'string' || lodash_1.isRegExp(arg));
const patchMethod = (app, method, converters) => {
    const old = app[method].bind(app);
    return ((...args) => {
        const first = args[0];
        if (method !== 'use' && !isPath(first)) {
            throw new Error('First middleware MUST BE a path/RegExp string');
        }
        const path = isPath(first) ? [first] : [];
        const argMiddlewares = isPath(first) ? args.slice(1) : args;
        const flattenMiddlewares = lodash_1.flattenDeep(argMiddlewares);
        const context = { method, path: path[0] };
        const middlewaresConverted = converters.reduce((middlewaresAcc, converter) => converter(middlewaresAcc, context), flattenMiddlewares);
        const middlewares = [
            ...path,
            ...middlewaresConverted.filter(types_1.isMiddlewareArg),
        ].filter(m => !!m);
        return old(...middlewares);
    });
};
const METHODS = [
    'get',
    'post',
    'put',
    'delete',
    'patch',
    'use',
];
const asyncAppProvider = (opts) => {
    const app = express_1.default();
    const converters = opts && opts.converters || [];
    if (opts && opts.compileSchemaFn) {
        converters.push(schema_1.default(opts.compileSchemaFn, opts.generateSchemaErrorFn));
    }
    const async = async_1.default({
        compileSchema: opts && opts.compileSchemaFn && opts.validateResponseSchema
            ? opts.compileSchemaFn
            : undefined,
        errorHandler: opts && opts.errorHandlerFn ? opts.errorHandlerFn : undefined,
        mapAsyncResultFn: opts && opts.mapAsyncResultFn,
    });
    METHODS.forEach(m => app[m] = patchMethod(app, m, [
        async,
        ...converters,
        order_1.default(),
    ]));
    return app;
};
let appProvider = asyncAppProvider;
exports.resetProvider = () => {
    appProvider = asyncAppProvider;
};
exports.setProvider = (provider) => {
    appProvider = provider;
};
exports.createApp = (opts) => appProvider(opts);
exports.default = exports.createApp;
//# sourceMappingURL=create-app.js.map