"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const decorate_1 = require("../decorate");
const error_1 = require("../error");
exports.loadFromRequest = (getModelFromRequest, storeModelInto, loadAdditionalModels) => {
    // Async Fn to fetch and store the model.
    const loadModel = async (req) => {
        const model = await getModelFromRequest(req);
        if (!model)
            throw error_1.notFound(`${storeModelInto}`.toUpperCase());
        req[storeModelInto] = model;
        // Load additional models i.e workspace
        if (loadAdditionalModels) {
            await loadAdditionalModels.fn(req, model);
        }
    };
    // Decorate properties.
    // Add $provides and $requires to order middlewares.
    const $provides = loadAdditionalModels
        ? [`${storeModelInto}`, loadAdditionalModels.key]
        : [`${storeModelInto}`];
    const $requires = getModelFromRequest.$requires || [];
    // Return decorated fn with $provides and $requires;
    return decorate_1.decorate({ $provides, $requires }, loadModel);
};
exports.loadOnlyOnce = (getIdFromModel) => (getModelFromRequest, getIdFromRequest, storeModelInto) => {
    const load = async (req) => {
        const id = getIdFromRequest(req);
        const oldModel = req[storeModelInto];
        // Invalid model stored in [storeModelInto] key
        if (oldModel && getIdFromModel(oldModel) !== id) {
            throw error_1.internalServerError('INVALID_MODEL_ID', {
                actualId: getIdFromModel(oldModel),
                expectedId: id,
                modelTarget: storeModelInto,
            });
        }
        return oldModel || await getModelFromRequest(req);
    };
    load.$requires = getIdFromRequest.$requires;
    return load;
};
exports.loadWith = (fetch, getIdFromModel) => {
    const load = exports.loadOnlyOnce(getIdFromModel);
    return (getIdFromRequest, storeModelInto, loadAdditionalModels) => exports.loadFromRequest(load(req => fetch(getIdFromRequest(req)), getIdFromRequest, storeModelInto), storeModelInto, loadAdditionalModels);
};
exports.loadOnceWith = (getIdFromModel) => {
    const load = exports.loadOnlyOnce(getIdFromModel);
    return (getModelFromRequest, getModelIdFromRequest, storeModelInto, requiredModels, loadAdditionalModels) => decorate_1.requires(requiredModels || getModelFromRequest.$requires || [], exports.loadFromRequest(load(getModelFromRequest, getModelIdFromRequest, storeModelInto), storeModelInto, loadAdditionalModels));
};
//# sourceMappingURL=index.js.map