"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const decorate_1 = require("../decorate");
const error_1 = require("../error");
const get_argument_obj_keys_1 = __importDefault(require("./get-argument-obj-keys"));
const to_string_1 = __importDefault(require("./to-string"));
const mapKey = (mapping, modelKey) => `${mapping[modelKey] || modelKey}`;
exports.createPermissionFor = (validator, permission) => {
    const expectedModels = get_argument_obj_keys_1.default(validator);
    const fn = (mapping = {}) => decorate_1.decorate({
        $permission: permission,
        $requires: expectedModels.map(key => mapKey(mapping, key)),
    }, (req, _, next) => {
        const args = {};
        expectedModels.forEach((key) => {
            const modelKey = mapKey(mapping, key);
            if (!req.hasOwnProperty(modelKey)) {
                throw new Error(`
            Permission "${permission}" middleware expected \`req.${modelKey}\`
            to be defined. Consider adding a \`load.${key}.fromX\` middleware
            before "${permission}".
          `);
            }
            args[key] = req[modelKey];
        });
        if (validator(args))
            return next();
        throw error_1.forbidden(to_string_1.default(permission));
    });
    return fn;
};
//# sourceMappingURL=create-middleware.js.map