"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const create_middleware_1 = require("./create-middleware");
const types_1 = require("./types");
const util_1 = require("./util");
const permissionContainer = (permission) => {
    const fn = (() => {
        throw new Error(`Invalid permission: \`${permission}\`, this is just a a container`);
    });
    fn.$container = true;
    return fn;
};
exports.createPermissions = (entities) => {
    const can = {};
    util_1.getKeys(entities).forEach((entityName) => {
        const entity = entities[entityName];
        lodash_1.mapValues(entity, (spec1, action) => {
            const spec = spec1;
            // load permission on entire entity, e.g.: `can.view.mural`
            const canAction = can[action]
                || (can[action] = {});
            const permission = `${entityName}.${action}`;
            const canItem = types_1.isPermissionFn(spec)
                ? create_middleware_1.createPermissionFor(spec, permission)
                : permissionContainer(permission);
            canAction[entityName] = canItem;
            // load subpermissions, e.g.: `can.manage.mural.editHash`
            if (types_1.isPermissionEntity(spec)) {
                lodash_1.mapValues(spec, (subspec, subaction) => {
                    if (types_1.isPermissionFn(subspec)) {
                        canItem[subaction] = create_middleware_1.createPermissionFor(subspec, `${entityName}.${action}.${subaction}`);
                    }
                });
            }
        });
    });
    return can;
};
//# sourceMappingURL=create.js.map