"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("./types");
exports.METHOD_SOURCE_MAP = {
    delete: 'query',
    get: 'query',
    patch: 'body',
    post: 'body',
    put: 'body',
};
const defaultGenerateError = ([error], source) => ({
    error: 'INVALID_PAYLOAD',
    expected: error.expected,
    path: error.key,
    source,
});
const createMiddleware = (validate, generateError, source, context) => (req, res, next) => {
    const realSource = source || exports.METHOD_SOURCE_MAP[req.method.toLowerCase()];
    const data = req[realSource];
    const schemaErrors = validate(data);
    if (schemaErrors.length) {
        res.status(400).send(generateError(schemaErrors, realSource, Object.assign({}, context, { req })));
        return;
    }
    return next();
};
exports.default = (compileSchema, generateError = defaultGenerateError) => (middlewares, context) => {
    const schema = middlewares
        .filter(types_1.isSchema())
        .find(s => s.$scope !== 'response');
    if (!schema)
        return middlewares;
    const source = schema.$scope || exports.METHOD_SOURCE_MAP[context.method];
    if (source === 'response')
        return middlewares;
    const schemaMiddleware = createMiddleware(compileSchema(schema.$schema || schema, context), generateError, source, context);
    schemaMiddleware.$noOrder = true;
    return [schemaMiddleware, ...middlewares];
};
//# sourceMappingURL=schema.js.map